classdef BVP < handle
  properties %(SetAccess = protected)
    Th = [];
    pdes = [];
    system=[];
    dim = 0;
    d=0;
    m = 0;
    ndof = 0;
    sym = false;
    Num = 1;
    name= '';
    AssemblyVersion='OptV3';
    info = [];
  end

  methods
    function self=BVP(Th,pde,labels)
      if nargin==0, return;end
      assert( fc_simesh.is_siMesh(Th) )
      assert( fc_vfemp1.is_PDE(pde) )
      assert( Th.dim == pde.dim);
      if nargin==2, idxlab=Th.find(pde.d);
      else idxlab=Th.find(pde.d,labels);end
      self.dim=pde.dim;
      self.d=Th.d;
      self.m=pde.m; % Scalar for the moment
      self.Th=Th;
      self.system=struct('A',[],'b',[],'ID',[],'IDc',[],'gD',[]);
      self.ndof=self.m*Th.nq; % For P1-Lagrange only!
      self.pdes=cell(1,Th.nsTh);
      for i=idxlab, self.pdes{i}=pde;end
    end
    
    function [Aout,bout]=Assembly(self,varargin)
      % A ameliorer!!!
      p = inputParser;
      p.addParamValue('Num',1,@(x) ismember(x,[0,1]));
      p.addParamValue('local',true,@islogical);
      p.addParamValue('physical',true,@islogical);
      p.addParamValue('interface',false,@islogical);
      p.addParamValue('Robin',true,@islogical); % Change in Robin
      p.addParamValue('Dirichlet',true,@islogical);
      p.addParamValue('dom',true,@islogical);
      %p.addParamValue('nq',self.Th.nq,@isscalar);
      p.parse(varargin{:});
      R=p.Results;

      A=spalloc(double(self.ndof),double(self.ndof),double(5*self.ndof));
      b=zeros(self.ndof,1);
      VFInd=fc_vfemp1.FEM.getVFindices(R.Num,self.m,self.Th.nq);
      if R.dom
        idxlab=self.Th.find(self.Th.d);
        [A,b]=fc_vfemp1.BVPlib.Assembly_main(self,A,b,VFInd,idxlab,R.local);
      end
      
      if R.Robin 
        [A,b]=fc_vfemp1.BVPlib.Assembly_BC(self,A,b,VFInd,R,@(varargin) fc_vfemp1.BVPlib.Assembly_Robin(varargin{:}));
      end
      if R.Dirichlet 
        [A,b]=fc_vfemp1.BVPlib.Assembly_BC(self,A,b,VFInd,R,@(varargin) fc_vfemp1.BVPlib.Assembly_Dirichlet(varargin{:}));
      end
      
    %    for dd=self.Th.d-1:-1:0 % 
    %      Allidxlab=self.Th.find(dd);
    %      if ~isempty(Allidxlab)
    %        idxlab=[];
    %        if R.physical
    %          I=find((self.Th.sThlab(Allidxlab)>=0) & (self.Th.sThlab(Allidxlab)<10000));
    %          idxlab=[idxlab,Allidxlab(I)];
    %        end
    %        if R.interface
    %          I=find((self.Th.sThlab(Allidxlab)<0)|(self.Th.sThlab(Allidxlab)>=10000));
    %          idxlab=[idxlab,Allidxlab(I)];
    %        end
    %        if R.Robin, [A,b]=Assembly_Robin(self,A,b,VFInd,idxlab);end
    %  
    %        if R.Dirichlet, [A,b]=Assembly_Dirichlet(self,A,b,VFInd,idxlab);end % ATTENTION BUG POSSIBLE : FAIRE ROBIN PUIS DIRICHLET
    %      end
    %    end
      
      if ~R.local
        nq=self.Th.nqParents(1);idx=self.Th.toParents{1};
        Aout=sparse(nq,nq);bout=zeros(nq,1);
        Aout(idx,idx)=A;
        bout(idx)=b;
    %      Aout=sparse(R.nq,R.nq);bout=zeros(R.nq,1);
    %      Aout(self.Th.toGlobal,self.Th.toGlobal)=A;
    %      bout(self.Th.toGlobal)=b;
      else
        Aout=A;
        bout=b;
      end
    end    
    
    function self=setDirichlet(self,label,fun,Lm)
      assert(length(label)==1,'Only a scalar label is allowed')
      idxlab=self.Th.find(self.d-1,label);
      if length(idxlab)==1
        LopD=fc_vfemp1.Loperator(self.dim,self.d-1,[],[],[],1);
        if nargin<=3
          Lm=1:self.m;
        end
        if isempty(self.pdes{idxlab})
          %self.pdes{idxlab}=PDE.zero(self.dim,self.d-1,self.m);
          self.pdes{idxlab}=fc_vfemp1.PDE();
          self.pdes{idxlab}=self.pdes{idxlab}.zeros(self.dim,self.d-1,self.m);
        end
        if self.m==1
          self.pdes{idxlab}.Op=LopD;
          self.pdes{idxlab}.f=fc_vfemp1.Fdata(fun);
          self.pdes{idxlab}.delta=0;
        else
          i=1;
          for l=Lm
            self.pdes{idxlab}.delta(l)=0;
            for k=1:self.m % To improve!!!
              if k~=l
                self.pdes{idxlab}.Op.H{l,k}=[];
              else
                self.pdes{idxlab}.Op.H{l,l}=LopD;
              end
            end
            if iscell(fun)
              self.pdes{idxlab}.f{l}=fc_vfemp1.Fdata(fun{i});
            else
              self.pdes{idxlab}.f{l}=fc_vfemp1.Fdata(fun);
            end
            i=i+1;
          end
        end
      else
        warning('label %d not found',label)
      end 
    end % function setDirichlet
    
    function self=setRobin(self,label,fun,ar,Lm)
      assert(length(label)==1,'Only a scalar label is allowed')
      idxlab=self.Th.find(self.d-1,label);
      if length(idxlab)==1
      %assert(length(idxlab)==1)
        if ~isempty(ar)
          LopD=fc_vfemp1.Loperator(self.dim,self.d-1,[],[],[],ar);
        else
          LopD=[];
        end
        if nargin<=4
          Lm=1:self.m;
        end
        if isempty(self.pdes{idxlab})
          self.pdes{idxlab}=fc_vfemp1.PDE();
          self.pdes{idxlab}=self.pdes{idxlab}.zeros(self.dim,self.d-1,self.m);
        end
        if self.m==1
          self.pdes{idxlab}.Op=LopD;
          self.pdes{idxlab}.f=fc_vfemp1.Fdata(fun);
          return
        else
          i=1;
          for l=Lm
            self.pdes{idxlab}.delta(l)=1;
            for k=1:self.m
              if k~=l
                self.pdes{idxlab}.Op.H{l,k}=[];
              else
                self.pdes{idxlab}.Op.H{l,l}=LopD;
              end
            end
            if iscell(fun)
              self.pdes{idxlab}.f{l}=fc_vfemp1.Fdata(fun{i});
            elseif fc_tools.utils.isfunhandle(fun)
              self.pdes{idxlab}.f{l}=fc_vfemp1.Fdata(fun);
            elseif size(fun,1)==length(Lm)
              self.pdes{idxlab}.f{l}=fc_vfemp1.Fdata(fun(i,:));
            elseif size(fun,2)==length(Lm)
              self.pdes{idxlab}.f{l}=fc_vfemp1.Fdata(fun(:,i));
            else
              self.pdes{idxlab}.f{l}=fc_vfemp1.Fdata(fun);
            end
            i=i+1;
          end
        end
      end  
    end % function setRobin  
    
%%%% To suppress ??? ->
    function self=setGeneralizedBC(self,level,label,delta,op,fun,varargin)
      p = inputParser;
      p.addParamValue('m',1:self.m);%,@(x) ismember(x,0:self.d));
      %p.addParamValue('d',self.d-1,@(x) ismember(x,0:self.d));
      %p.addParamValue('op',[],@(x) ismember(class(x),{'function_handle','double','Loperator'}));
      p.parse(varargin{:});
      d=self.Th.d-level;Lm=p.Results.m;%op=p.Results.op;
      idxlab=self.Th.find(d,label);
      if length(idxlab)==1
      %assert(length(idxlab)==1)
        if ~isempty(op)
          if fc_vfemp1.is_Loperator(op)
            LopD=op;
          else
            LopD=fc_vfemp1.Loperator(self.dim,d,[],[],[],op);
          end
        else
          LopD=[];
        end
        if nargin<=4
          Lm=1:self.m;
        end
        if isempty(self.pdes{idxlab})
          self.pdes{idxlab}=fc_vfemp1.PDE();
          self.pdes{idxlab}=self.pdes{idxlab}.zeros(self.dim,d,self.m);
        end
        i=1;
        for l=Lm
          self.pdes{idxlab}.delta(l)=delta;
          for k=1:self.m
            if k~=l
              self.pdes{idxlab}.Op.H{l,k}=[];
            else
              self.pdes{idxlab}.Op.H{l,l}=LopD;
            end
          end
          if iscell(fun)
            self.pdes{idxlab}.f{l}=fc_vfemp1.Fdata(fun{i});
          else
            self.pdes{idxlab}.f{l}=fc_vfemp1.Fdata(fun);
          end
          i=i+1;
        end
      end  
    end
      
    function self=setPDE(self,d,label,pde)
      assert( fc_vfemp1.is_PDE(pde) )
      idxlab=self.Th.find(d,label);
      if length(idxlab)==1
        self.pdes{idxlab}=pde;
      end
    end
  
    function varargout=solve(self,varargin) 
    % Optional key/value parameters:
    %   'solver' :
    %         @(A,b) gmres(A,b,10,1e-12,100)
    %      default, @mldivide
      p = inputParser;
      p.addParamValue('solver',@mldivide,@(f) strcmp(class(f),'function_handle'));
      p.addParamValue('perm',[],@(f) strcmp(class(f),'function_handle') || isempty(f));
      p.addParamValue('local',true,@islogical);
      p.addParamValue('verbose',0,@isscalar);
      p.addParamValue('split',false,@islogical);
      p.addParamValue('time',false,@islogical);
      p.addParamValue('method',1,@(x) ismember(x,1:2)); % 1: 
                                                        % 2: with relevement                           
      %p.addParamValue('info',false,@islogical);
      p.parse(varargin{:});
      R=p.Results;
      method=R.method;
      funsolve=p.Results.solver;
      perm=p.Results.perm;
      switch method
        case 1
          tstart=tic();  
          [Ag,bg]=self.Assembly('local',p.Results.local);  
          tcpu(1)=toc(tstart);
        case 2
          tstart=tic();  
          [Ag,bg]=self.Assembly('local',p.Results.local,'Dirichlet',false);
          [ID,IDc,gD]=self.AssemblyDirichlet();
          tcpu(1)=toc(tstart);
      end
      tname{1}='Assembly';
      nout_solve=nargout-p.Results.time;
      assert(nout_solve>=1,'With time option 2 output are required')
      
      if fc_tools.comp.isOctave(), varargout=cell(1,nargout); end % Needed by Octave
      if isempty(perm)
        switch method
          case 1
            tstart=tic();  varargout{1:nout_solve}=funsolve(Ag,bg);  tcpu(2)=toc(tstart);
          case 2
            tstart=tic();  
            varargout{1:nout_solve}=fc_vfemp1.FEM.SolveWithExtension(Ag,bg,gD,ID,IDc,'solver',funsolve);
            tcpu(2)=toc(tstart);
        end
    %        res=norm(Ag*varargout{1}-bg,Inf);
    %        if p.Results.verbose>0
    %          fprintf('[fc-vfemp1] BVP.solve, residu : %.16f\n',res)
    %        end
      else
        switch method
          case 1
            tstart=tic();
            pe=perm(Ag);
            varargout{1:nout_solve}=funsolve(Ag(pe,pe),bg(pe));
            varargout{1}(pe)=varargout{1};
            tcpu(2)=toc(tstart);
          case 2
            tstart=tic();
            pe=perm(Ag); [~,idx]=sort(pe);
            [varargout{1:nout_solve}]=fc_vfemp1.FEM.SolveWithExtension(Ag(pe,pe),bg(pe),gD(pe),idx(ID),idx(IDc),'solver',funsolve);
            varargout{1}(pe)=varargout{1};
            tcpu(2)=toc(tstart);
        end
      end
      tname{2}='Solve';

      if p.Results.time
        solveInfo.tcpu=tcpu;solveInfo.tname=tname;solveInfo.solver=fc_tools.utils.fun2str(funsolve);
        solveInfo.matrixsize=size(Ag);solveInfo.residu=norm(Ag*varargout{1}-bg,Inf);
        solveInfo.nnz=nnz(Ag);
    %    isfield(info
        
        varargout{nargout}=solveInfo;
      end
      if p.Results.split
        varargout{1}=self.splitsol(varargout{1});
      end
    end
    
    function x=splitsol(self,X,Num)
    % aaa
      if nargin==2, Num=self.Num;end
      if self.m==1, x=X; return; end
      VFInd=fc_vfemp1.FEM.getVFindices(Num,self.m,self.Th.nq);
      x=cell(self.m,1);
      I=1:self.Th.nq;
      for i=1:self.m
        x{i}=X(VFInd(I,i));
      end
    end
    
    function [ID,IDc,gD]=AssemblyDirichlet(self,varargin)
      p = inputParser;
      p.addParamValue('Num',1,@(x) ismember(x,[0,1]));
      p.parse(varargin{:});
      R=p.Results;
      gD=zeros(self.ndof,1);
      ID=[];
      VFInd=fc_vfemp1.FEM.getVFindices(R.Num,self.m,self.Th.nq);
      idxlab=self.Th.find(self.Th.d-1);
      for i=idxlab
        if ~isempty(self.pdes{i})  
          tg=self.Th.sTh{i}.toParent;
          if size(tg,1)>1, tg=tg';end
          for l=1:self.m
            if self.pdes{i}.delta(l)==0 % Dirichlet 
              tgl=VFInd(tg,l);
              if self.m==1 % FC : to improve
                gD(tgl)=self.pdes{i}.f.eval(self.Th.sTh{i});
              else
                gD(tgl)=self.pdes{i}.f{l}.eval(self.Th.sTh{i});
              end
              ID=[ID,tgl];
            end
          end
        end 
      end
      ID=unique(ID);
      IDc=setdiff(1:self.ndof,ID);
    end

  end % methods
end % classdef


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  function [A,b]=Assembly_main(self,A,b,VFInd,idxlab,local)
%    for i=idxlab
%      if ~isempty(self.pdes{i})
%        [Ai,bi]=self.pdes{i}.Assembly(self.Th.sTh{i},'local',local,'block',true);
%        for l=1:self.m
%          tgl=VFInd(self.Th.sTh{i}.toParent,l);
%          for s=1:self.m
%            tgs=VFInd(self.Th.sTh{i}.toParent,s);
%            if ~isempty(Ai{l,s}), A(tgl,tgs)=A(tgl,tgs)+Ai{l,s};end
%          end
%          if ~isempty(bi{l}), b(tgl)=b(tgl)+bi{l};end
%        end
%      end 
%    end
%  end
%  
%  function [A,b]=Assembly_BC(self,A,b,VFInd,R,Assemblyfun);
%    for dd=self.Th.d-1:-1:0 % 
%      Allidxlab=self.Th.find(dd);
%      if ~isempty(Allidxlab)
%        idxlab=[];
%        if R.physical
%          I=find((self.Th.sThlab(Allidxlab)>=0) & (self.Th.sThlab(Allidxlab)<10000));
%          idxlab=[idxlab,Allidxlab(I)];
%        end
%        if R.interface
%          I=find((self.Th.sThlab(Allidxlab)<0)|(self.Th.sThlab(Allidxlab)>=10000));
%          idxlab=[idxlab,Allidxlab(I)];
%        end
%        if ~isempty(idxlab)
%          [A,b]=Assemblyfun(self,A,b,VFInd,idxlab);
%        end
%      end
%    end
%  end
%  
%  function [A,b]=Assembly_Robin(self,A,b,VFInd,idxlab)
%    for i=idxlab 
%      if ~isempty(self.pdes{i})
%        [Ai,bi]=self.pdes{i}.Assembly(self.Th.sTh{i},'block',true);
%        for l=1:self.m
%          if self.pdes{i}.delta(l)==1 % Robin
%            tgl=VFInd(self.Th.sTh{i}.toParent,l);   
%            for k=1:self.m
%              if ~isempty(Ai{l,k})
%                tgk=VFInd(self.Th.sTh{i}.toParent,k);    
%                A(tgl,tgk)=A(tgl,tgk)+Ai{l,k};
%              end
%            end
%            if ~isempty(bi{l})
%                b(tgl)=b(tgl)+bi{l};
%            end         
%          end
%        end
%      end
%    end
%  end
%  
%  function [A,b]=Assembly_Dirichlet(self,A,b,VFInd,idxlab)
%    for i=idxlab % Puis Dirichlet
%      if ~isempty(self.pdes{i})
%        [Ai,bi]=self.pdes{i}.Assembly(self.Th.sTh{i},'block',true);
%        for l=1:self.m
%          if self.pdes{i}.delta(l)==0 % Dirichlet
%            tgl=VFInd(self.Th.sTh{i}.toParent,l);
%            for k=1:self.m
%              if ~isempty(Ai{l,k})
%                tgk=VFInd(self.Th.sTh{i}.toParent,k);
%                A(tgl,:)=0;
%                A(tgl,tgk)=Ai{l,k};
%              end
%            end
%            if ~isempty(bi{l}), b(tgl)=bi{l};end
%          end
%        end
%      end
%    end
%  end
%  
%  
